@@ -319,7 +319,8 @@ class Options(dict):
                             for kw in ('first_number', 'line_numbers', 'rewrap_lines', 'rewrap_width', 'expand_tabs', 'tab_size')])
     _first_chunk_execute_keywords = set(['executable', 'jupyter_kernel'])
     _first_chunk_save_keywords = set(['save', 'save_as'])
-    _first_chunk_keywords = _first_chunk_execute_keywords | _first_chunk_save_keywords
+    _first_chunk_other_keywords = set(['jupyter_timeout', 'live_output'])
+    _first_chunk_keywords = _first_chunk_execute_keywords | _first_chunk_save_keywords | _first_chunk_other_keywords
 
     keywords = _base_keywords | _layout_keywords | _first_chunk_keywords
 
@@ -361,6 +362,7 @@ class Options(dict):
                                                                   ('stderr', 'verbatim'),
                                                                   ('rich_output', _default_rich_output)]),
                                                    'paste': ODict(),
+                                                   'repl': ODict([('repl', 'verbatim')]),
                                                    'run':  ODict([('stdout', 'raw'),
                                                                   ('stderr', 'verbatim'),
                                                                   ('rich_output', _default_rich_output)])})
@@ -420,10 +422,18 @@ class Options(dict):
             else:
                 first_chunk_options[key] = value
 
+    def _option_first_chunk_int_warning(self, key, value):
+        if not isinstance(value, int):
+            self.code_chunk.source_warnings.append('Invalid "{0}" value "{1}"'.format(key, value))
+        else:
+            self['first_chunk_options'][key] = value
+
     _option_executable = _option_first_chunk_string_error
     _option_jupyter_kernel = _option_first_chunk_string_error
+    _option_jupyter_timeout = _option_first_chunk_int_warning
     _option_save = _option_first_chunk_bool_error
     _option_save_as = _option_first_chunk_string_error
+    _option_live_output = _option_first_chunk_bool_error
 
     _option_example = _option_bool_warning
     _option_lang = _option_str_error
@@ -434,6 +444,8 @@ class Options(dict):
             self.code_chunk.source_errors.append('Invalid "{0}" value "{1}"'.format(key, value))
         elif not self.code_chunk.execute:
             self.code_chunk.source_errors.append('Option "complete" is only compatible with executed code chunks')
+        elif self.code_chunk.command == 'repl':
+            self.code_chunk.source_errors.append('Option "complete" is not compatible with "repl" command')
         elif self.code_chunk.is_expr and not value:
             self.code_chunk.source_errors.append('Option "complete" value "false" is incompatible with expr command')
         elif self['outside_main']:
@@ -578,6 +590,8 @@ class Options(dict):
             self.code_chunk.source_errors.append('Invalid "{0}" value "{1}"'.format(key, value))
         elif not self.code_chunk.execute:
             self.code_chunk.source_errors.append('Option "outside_main" is only compatible with executed code chunks')
+        elif self.code_chunk.command == 'repl':
+            self.code_chunk.source_errors.append('Option "outside_main" is not compatible with "repl" command')
         elif self.code_chunk.is_expr and value:
             self.code_chunk.source_errors.append('Option "outside_main" value "true" is incompatible with expr command')
         elif value and 'complete' in self.custom_options:
@@ -623,6 +637,20 @@ class Options(dict):
                 'svg': 'image/svg+xml',
                 'pdf': 'application/pdf'}
 
+    mime_map_with_text_display = {}
+    rich_text_default_display = {}
+    for k, v in mime_map.items():
+        mime_map_with_text_display[k] = v
+        if v.startswith('text/'):
+            mime_map_with_text_display[k+':raw'] = v
+            mime_map_with_text_display[k+':verbatim'] = v
+            mime_map_with_text_display[k+':verbatim_or_empty'] = v
+            if k == 'plain':
+                rich_text_default_display[k] = 'verbatim'
+            else:
+                rich_text_default_display[k] = 'raw'
+
+
     def _option_show(self, key, value):
         # 'show' may be processed during `finalize_after_copy()` to allow for
         # 'show' and `.is_expr` inheritance.  'hide' checks for 'show'
@@ -642,7 +670,7 @@ class Options(dict):
                 if output in value_processed:
                     self.code_chunk.source_warnings.append('Option "{0}" value "{1}" contains duplicate "{2}"'.format(key, value, output))
                     continue
-                if output in ('markup', 'copied_markup', 'code'):
+                if output in ('markup', 'copied_markup', 'code', 'repl'):
                     if format is None:
                         format = 'verbatim'
                     elif format != 'verbatim':
@@ -671,7 +699,7 @@ class Options(dict):
                         format = self._default_rich_output
                     else:
                         format = format.split('|')
-                        if not all(fmt in self.mime_map for fmt in format):
+                        if not all(fmt in self.mime_map_with_text_display for fmt in format):
                             self.code_chunk.source_warnings.append('Invalid "{0}" sub-value "{1}"'.format(key, output_and_format))
                             continue
                 else:
@@ -718,6 +746,8 @@ class CodeChunk(object):
             self.command = command
         if command == 'expr' and not inline:
             self.source_errors.append('Codebraid command "{0}" is only allowed inline'.format(command))
+        if command == 'repl' and inline:
+            self.source_errors.append('Codebraid command "{0}" is not supported inline'.format(command))
         self.execute = self._default_execute[command]
         if command == 'expr' or (inline and command == 'nb'):
             self.is_expr = True
@@ -782,6 +812,7 @@ class CodeChunk(object):
             self.source_index = None
         self.stdout_lines = None
         self.stderr_lines = None
+        self.repl_lines = None
         self.rich_output = None
         if self.is_expr:
             self.expr_lines = None
@@ -803,10 +834,10 @@ class CodeChunk(object):
             self.source_warnings = []
 
 
-    commands = set(['code', 'expr', 'nb', 'run', 'paste'])
+    commands = set(['code', 'expr', 'nb', 'paste', 'repl', 'run'])
 
     _default_execute = collections.defaultdict(lambda: False,  # Unknown command -> do not run
-                                               {k: True for k in ('expr', 'nb', 'run')})
+                                               {k: True for k in ('expr', 'nb', 'repl', 'run')})
 
 
     @property
@@ -911,13 +942,15 @@ class CodeChunk(object):
         if len(copy_chunks) == 1:
             self.stdout_lines = copy_chunks[0].stdout_lines
             self.stderr_lines = copy_chunks[0].stderr_lines
+            self.repl_lines = copy_chunks[0].repl_lines
             self.rich_output = copy_chunks[0].rich_output
         else:
             self.stdout_lines = [line for x in copy_chunks if x.stdout_lines is not None for line in x.stdout_lines] or None
             self.stderr_lines = [line for x in copy_chunks if x.stderr_lines is not None for line in x.stderr_lines] or None
+            self.repl_lines = [line for x in copy_chunks if x.repl_lines is not None for line in x.repl_lines] or None
             self.rich_output = [ro for x in copy_chunks if x.rich_output is not None for ro in x.rich_output] or None
         if self.is_expr:
-            # expr compatibilty has already been checked in `copy_code()`
+            # expr compatibility has already been checked in `copy_code()`
             self.expr_lines = copy_chunks[0].expr_lines
         self.stdout_start_line_number = copy_chunks[0].stdout_start_line_number
         self.stderr_start_line_number = copy_chunks[0].stderr_start_line_number
@@ -932,9 +965,13 @@ class CodeChunk(object):
         default lines (if any) are accessed for the specified output type.
         '''
         if lines is not None:
+            if not lines and output_format == 'verbatim_or_empty':
+                lines = ['\xa0']
             pass
         elif output_type == 'code':
             lines = self.code_lines
+        elif output_type == 'repl':
+            lines = self.repl_lines
         elif output_type in ('expr', 'stdout', 'stderr'):
             lines = getattr(self, output_type+'_lines')
             if lines is None and output_format == 'verbatim_or_empty':
