@@ -18,6 +18,7 @@ import re
 import subprocess
 import sys
 import tempfile
+import textwrap
 from typing import Dict, List, Optional, Sequence, Union
 import warnings
 from .base import CodeChunk, Converter, Include
@@ -183,8 +184,10 @@ def _get_keyval_processors():
                                     **expand_tabs,
                                     **first_number,
                                     **include,
+                                    'jupyter_timeout': keyval_int,
                                     **line_anchors,
                                     **line_numbers,
+                                    'live_output': keyval_bool,
                                     'name': keyval_generic,
                                     'outside_main': keyval_bool,
                                     **rewrap_lines,
@@ -244,8 +247,13 @@ class PandocCodeChunk(CodeChunk):
         pandoc_id = node_id
         pandoc_classes = []
         pandoc_kvpairs = []
-        if 'lang' in options:
-            pandoc_classes.append(options['lang'])
+        lang = options.get('lang', None)
+        if lang is not None:
+            if lang.endswith('_repl'):
+                lang = lang.rsplit('_')[0]
+            pandoc_classes.append(lang)
+            if self.command == 'repl':
+                pandoc_classes.append('repl')
         if line_anchors:
             pandoc_classes.append('lineAnchors')
         if self.options.get('code_line_numbers', False):
@@ -306,66 +314,94 @@ class PandocCodeChunk(CodeChunk):
             self.pandoc_kvpairs.append(['startFrom', first_number])
         t_code = 'Code' if self.inline else 'CodeBlock'
         t_raw = 'RawInline' if self.inline else 'RawBlock'
+        unformatted_nodes = []
         for output, format in self.options['show'].items():
-            new_nodes = None
             if output in ('markup', 'copied_markup'):
-                new_nodes = [{'t': t_code, 'c': [['', ['markdown'], []], self.layout_output(output, format)]}]
+                unformatted_nodes.append({'t': t_code, 'c': [['', ['markdown'], []], self.layout_output(output, format)]})
             elif output == 'code':
-                new_nodes = [{'t': t_code, 'c': [[self.pandoc_id, self.pandoc_classes, self.pandoc_kvpairs], self.layout_output(output, format)]}]
+                unformatted_nodes.append({'t': t_code, 'c': [[self.pandoc_id, self.pandoc_classes, self.pandoc_kvpairs], self.layout_output(output, format)]})
+            elif output == 'repl':
+                if self.repl_lines is None:
+                    continue
+                unformatted_nodes.append({'t': t_code, 'c': [[self.pandoc_id, self.pandoc_classes, self.pandoc_kvpairs], self.layout_output(output, format)]})
             elif output in ('expr', 'stdout', 'stderr'):
                 if format == 'verbatim':
                     if getattr(self, output+'_lines') is not None:
-                        new_nodes = [{'t': t_code, 'c': [['', [output], []], self.layout_output(output, format)]}]
+                        unformatted_nodes.append({'t': t_code, 'c': [['', [output], []], self.layout_output(output, format)]})
                 elif format == 'verbatim_or_empty':
-                    new_nodes = [{'t': t_code, 'c': [['', [output], []], self.layout_output(output, format)]}]
+                    unformatted_nodes.append({'t': t_code, 'c': [['', [output], []], self.layout_output(output, format)]})
                 elif format == 'raw':
                     if getattr(self, output+'_lines') is not None:
-                        new_nodes = [{'t': t_raw, 'c': ['markdown', self.layout_output(output, format)]}]
+                        unformatted_nodes.append({'t': t_raw, 'c': ['markdown', self.layout_output(output, format)]})
                 else:
                     raise ValueError
             elif output == 'rich_output':
                 if self.rich_output is None:
                     continue
-                new_nodes = []
                 for ro in self.rich_output:
                     ro_data = ro['data']
                     ro_files = ro['files']
-                    for fmt in format:
+                    for fmt_and_text_display in format:
+                        if ':' in fmt_and_text_display:
+                            fmt, fmt_text_display = fmt_and_text_display.split(':')
+                        else:
+                            fmt = fmt_and_text_display
+                            fmt_text_display = self.options.rich_text_default_display.get(fmt)
                         fmt_mime_type = self.options.mime_map[fmt]
                         if fmt_mime_type not in ro_data:
                             continue
                         if fmt_mime_type in ro_files:
                             image_node = {'t': 'Image', 'c': [['', [], []], [], [ro_files[fmt_mime_type], '']]}
                             if self.inline:
-                                new_nodes.append(image_node)
+                                unformatted_nodes.append(image_node)
                             else:
                                 para_node = {'t': 'Para', 'c': [image_node]}
-                                new_nodes.append(para_node)
+                                unformatted_nodes.append(para_node)
                             break
                         data = ro_data[fmt_mime_type]
-                        if fmt == 'latex':
-                            raw_node = {'t': 'RawInline', 'c': ['tex', data]}
-                            if self.inline:
-                                new_nodes.append(raw_node)
+                        lines = util.splitlines_lf(data)
+                        if fmt in ('latex', 'html', 'markdown'):
+                            if fmt == 'latex':
+                                raw_fmt = 'tex'
                             else:
-                                para_node = {'t': 'Para', 'c': [raw_node]}
-                                new_nodes.append(para_node)
-                            break
-                        if fmt in ('html', 'markdown'):
-                            raw_node = {'t': t_raw, 'c': [fmt, data]}
-                            new_nodes.append(raw_node)
+                                raw_fmt = fmt
+                            if fmt_text_display == 'raw':
+                                unformatted_nodes.append({'t': t_raw, 'c': [raw_fmt, self.layout_output(output, 'raw', lines)]})
+                            elif fmt_text_display == 'verbatim':
+                                if lines:
+                                    unformatted_nodes.append({'t': t_code, 'c': [['', [fmt], []], self.layout_output(output, 'verbatim', lines)]})
+                            elif fmt_text_display == 'verbatim_or_empty':
+                                unformatted_nodes.append({'t': t_code, 'c': [['', [fmt], []], self.layout_output(output, 'verbatim', lines)]})
+                            else:
+                                raise ValueError
                             break
                         if fmt == 'plain':
-                            lines = util.splitlines_lf(data) or None
-                            if lines is not None:
-                                code_node = {'t': t_code, 'c': [['', [], []], self.layout_output(output, 'verbatim', lines)]}
-                                new_nodes.append(code_node)
+                            if fmt_text_display == 'raw':
+                                unformatted_nodes.append({'t': t_raw, 'c': ['markdown', self.layout_output(output, fmt_text_display, lines)]})
+                            elif fmt_text_display == 'verbatim':
+                                if lines:
+                                    unformatted_nodes.append({'t': t_code, 'c': [['', [], []], self.layout_output(output, 'verbatim', lines)]})
+                            elif fmt_text_display == 'verbatim_or_empty':
+                                unformatted_nodes.append({'t': t_code, 'c': [['', [], []], self.layout_output(output, 'verbatim', lines)]})
+                            else:
+                                raise ValueError
                             break
                         raise ValueError
             else:
                 raise ValueError
-            if new_nodes is not None:
-                nodes.extend(new_nodes)
+        if unformatted_nodes:
+            # Prevent adjacent nodes from merging unintentionally when
+            # converted through intermediate Markdown
+            for node, next_node in zip(unformatted_nodes[:-1], unformatted_nodes[1:]):
+                nodes.append(node)
+                if node['t'] == t_raw or next_node['t'] == t_raw:
+                    if self.inline:
+                        nodes.append({'t': 'Str', 'c': ' '})
+                    else:
+                        nodes.append({'t': 'Para', 'c': []})
+                elif self.inline and node['t'] == next_node['t'] == t_code:
+                    nodes.append({'t': 'Str', 'c': ' '})
+            nodes.append(unformatted_nodes[-1])
         self._output_nodes = nodes
         return nodes
 
@@ -482,10 +518,14 @@ def _get_walk_closure(enumerate=enumerate, isinstance=isinstance):
             if isinstance(obj, list):
                 yield from walk_node_list(obj)
             elif isinstance(obj, dict):
+                try:
+                    node_type = obj['t']
+                except KeyError:
+                    continue
                 yield (obj, node_list, index)
                 obj_contents = obj.get('c', None)
                 if isinstance(obj_contents, list):
-                    if obj['t'] != 'DefinitionList':
+                    if node_type != 'DefinitionList':
                         yield from walk_node_list(obj_contents)
                     else:
                         for elem in obj_contents:
@@ -509,12 +549,16 @@ def _get_walk_less_note_contents_closure(enumerate=enumerate, isinstance=isinsta
             if isinstance(obj, list):
                 yield from walk_node_list_less_note_contents(obj)
             elif isinstance(obj, dict):
+                try:
+                    node_type = obj['t']
+                except KeyError:
+                    continue
                 yield (obj, node_list, index)
-                if obj['t'] == 'Note':
+                if node_type == 'Note':
                     continue
                 obj_contents = obj.get('c', None)
                 if isinstance(obj_contents, list):
-                    if obj['t'] != 'DefinitionList':
+                    if node_type != 'DefinitionList':
                         yield from walk_node_list_less_note_contents(obj_contents)
                     else:
                         for elem in obj_contents:
@@ -565,9 +609,29 @@ class PandocConverter(Converter):
         pandoc_version_match = re.search(rb'\d+\.\d+', proc.stdout)
         if not pandoc_version_match:
             raise RuntimeError('Could not determine Pandoc version from "{0} --version"; faulty Pandoc installation?'.format(pandoc_path))
-        elif float(pandoc_version_match.group()) < 2.4:
-            raise RuntimeError('Pandoc at "{0}" is version {1}, but >= 2.4 is required'.format(pandoc_path, float(pandoc_version_match.group())))
+        pandoc_version_major, pandoc_version_minor = (float(x) for x in pandoc_version_match.group().split(b'.', 1))
+        if pandoc_version_major < 2 or pandoc_version_minor < 4:
+            raise RuntimeError('Pandoc at "{0}" is version {1}.{2}, but >= 2.4 is required'.format(pandoc_path, pandoc_version_major, pandoc_version_minor))
         self.pandoc_path = pandoc_path
+        if platform.system() == 'Windows':
+            pandoc_template_path = pathlib.Path('~/AppData/Roaming/pandoc/templates').expanduser()
+        else:
+            pandoc_data_path = pathlib.Path(os.environ.get('XDG_DATA_HOME', '~/.local/share')).expanduser() / 'pandoc'
+            if not pandoc_data_path.is_dir():
+                pandoc_data_path = pathlib.Path('~/.pandoc').expanduser()
+            pandoc_template_path = pandoc_data_path / 'templates'
+        if not pandoc_template_path.is_dir():
+            pandoc_template_path.mkdir(parents=True)
+        codebraid_markdown_roundtrip_template_path = pandoc_template_path / 'codebraid_pandoc_markdown_roundtrip.md'
+        if not codebraid_markdown_roundtrip_template_path.is_file():
+            codebraid_markdown_roundtrip_template_path.write_text(textwrap.dedent('''\
+                $if(titleblock)$
+                $titleblock$
+
+                $endif$
+                $body$
+                '''), encoding='utf8')
+        self.codebraid_markdown_roundtrip_template_path = codebraid_markdown_roundtrip_template_path
 
         if not isinstance(pandoc_file_scope, bool):
             raise TypeError
@@ -643,8 +707,8 @@ class PandocConverter(Converter):
                     input_name: Optional[str]=None,
                     output_path: Optional[pathlib.Path]=None,
                     standalone: bool=False,
-                    trace: bool=False,
                     newline_lf: bool=False,
+                    preserve_tabs: bool=False,
                     other_pandoc_args: Optional[List[str]]=None):
         '''
         Convert between formats using Pandoc.
@@ -665,10 +729,15 @@ class PandocConverter(Converter):
             cmd_list.extend(['--to', to_format + to_format_pandoc_extensions])
         if standalone:
             cmd_list.append('--standalone')
-        if trace:
-            cmd_list.append('--trace')
+            if to_format is None:
+                if output_path is not None and output_path.suffix in ('.md', '.markdown'):
+                    cmd_list.extend(['--template', self.codebraid_markdown_roundtrip_template_path.as_posix()])
+            elif to_format == 'markdown':
+                cmd_list.extend(['--template', self.codebraid_markdown_roundtrip_template_path.as_posix()])
         if file_scope:
             cmd_list.append('--file-scope')
+        if preserve_tabs:
+            cmd_list.append('--preserve-tabs')
         if output_path:
             cmd_list.extend(['--output', output_path.as_posix()])
         if other_pandoc_args:
@@ -826,14 +895,6 @@ class PandocConverter(Converter):
         node['c'] = [raw_format, '\x02CodebraidTrace({0})\x03'.format(trace) + node['c'][1]]
 
 
-    # Regex for footnotes.  While they shouldn't be indented, the Pandoc
-    # parser does recognize them as long as the indentation is less than 4
-    # spaces.  The restrictions on the footnote identifier are minimal.
-    # https://pandoc.org/MANUAL.html#footnotes
-    # https://github.com/jgm/pandoc/blob/master/src/Text/Pandoc/Readers/Markdown.hs
-    _footnote_re = re.compile(r' {0,3}\[\^[^ \t]+?\]:')
-
-
     def _load_and_process_initial_ast(self, *,
                                       source_string, single_source_name=None,
                                       any=any, int=int, len=len, next=next):
@@ -857,40 +918,24 @@ class PandocConverter(Converter):
         # cases, since not all AST information like attributes is used in
         # searching, but it will usually give exact results.  Since syncing is
         # a usability feature, an occasional small loss of precision isn't
-        # critical.  Pandoc's `--trace` output is used to simplify this
-        # process.  The trace can be used to determine the exact start of
-        # root-level nodes, so it periodically corrects any errors that have
-        # accumulated.  It can also be used to locate and skip over footnotes
-        # and link definitions.  These do not appear in the AST except in
-        # processed form, so they would be a primary source of sync errors
-        # otherwise.
-        #
-        # The original attempt at implementing line number syncing used only
-        # the trace plus AST, with no string searching.  That might have had
-        # much better performance had it worked.  Unfortunately, the data in
-        # the trace is typically only sufficient to give approximate locations
-        # at best.  The Pandoc markdown extension markdown_in_html_blocks can
-        # produce markdown nodes preceded and followed by HTML RawBlocks, but
-        # only the following RawBlock appears in the trace.  Similarly, HTML
-        # RawBlocks can appear out of order in the trace (at the end of the
-        # current chunk, rather than where they appear relative to other leaf
-        # nodes; for example, `<hr/>`).  The trace data for DefinitionList
-        # follows different rules (line numbers typically larger by 1),
-        # presumably because of how Pandoc's parsing works.  Empty elements,
-        # such as might occur in an incomplete bullet list, are difficult to
-        # deal with.
+        # critical.
+
+        # Convert source string to AST with Pandoc.
+        # Order of extensions is important: earlier override later.
+        from_format_pandoc_extensions = ''.join(['-latex_macros',
+                                                 '-smart'])
+        if self.from_format_pandoc_extensions is not None:
+            from_format_pandoc_extensions += self.from_format_pandoc_extensions
 
-        # Convert source string to trace plus AST with Pandoc
-        from_format_pandoc_extensions = self.from_format_pandoc_extensions or ''
-        if self.from_format == 'markdown':
-            from_format_pandoc_extensions += '-latex_macros-smart'
         stdout_bytes, stderr_bytes = self._run_pandoc(input=source_string,
                                                       input_name=single_source_name,
                                                       from_format=self.from_format,
                                                       from_format_pandoc_extensions=from_format_pandoc_extensions,
                                                       to_format='json',
-                                                      trace=True,
-                                                      newline_lf=True)
+                                                      newline_lf=True,
+                                                      preserve_tabs=True)
+        if stderr_bytes:
+            sys.stderr.buffer.write(stderr_bytes)
         try:
             if sys.version_info < (3, 6):
                 ast = json.loads(stdout_bytes.decode('utf8'))
@@ -901,92 +946,21 @@ class PandocConverter(Converter):
         if not (isinstance(ast, dict) and
                 'pandoc-api-version' in ast and isinstance(ast['pandoc-api-version'], list) and
                 all(isinstance(x, int) for x in ast['pandoc-api-version']) and 'blocks' in ast):
-            raise PandocError('Incompatible Pandoc API version')
-        if ast['pandoc-api-version'][0:2] != [1, 17]:
-            warnings.warn('Pandoc API is {0}.{1}, but Codebraid is designed for 1.17; this might cause issues'.format(*ast['pandoc-api-version'][0:2]))
+            raise PandocError('Unrecognized AST format (incompatible Pandoc version?)')
         self._asts[single_source_name] = ast
 
-        source_string_lines = util.splitlines_lf(source_string) or ['']
-
-        # Process trace to determine location of footnotes and link
-        # definitions, and mark those line numbers as invalid
-        invalid_line_numbers = set()
-        left_trace_type_slice_index = len('[trace] Parsed [')
-        right_trace_chunk_slice_index = len(' of chunk')
-        footnote_re = self._footnote_re
-        in_footnote = False
-        trace = ('', 1, False)  # (<node type>, <line number>, <in chunk>)
-        # Decode stderr using universal newlines
-        stderr_str = io.TextIOWrapper(io.BytesIO(stderr_bytes), encoding='utf8').read()
-        try:
-            for trace_line in util.splitlines_lf(stderr_str):
-                if trace_line.startswith('[WARNING]'):
-                    print(trace_line, file=sys.stderr)
-                    continue
-                last_trace_node_type, last_trace_line_number, last_trace_in_chunk = trace
-                trace_line = trace_line[left_trace_type_slice_index:]
-                trace_node_type, trace_line = trace_line.split(' ', 1)
-                trace_node_type = trace_node_type.rstrip(']')
-                if trace_line.endswith('chunk'):
-                    trace_in_chunk = True
-                    trace_line_number = int(trace_line.split('at line ', 1)[1][:-right_trace_chunk_slice_index])
-                else:
-                    trace_in_chunk = False
-                    trace_line_number = int(trace_line.split('at line ', 1)[1])
-                trace = (trace_node_type, trace_line_number, trace_in_chunk)
-                # Since footnotes must be at root level in the AST, line
-                # numbers are guaranteed to be correct for their start
-                # locations (no chunk numbering involved), so this will always
-                # detect and skip them correctly.  There shouldn't be any
-                # indentation before the `[^<identifier>]:`, but a regex is
-                # used to cover all cases that the parser will accept.  A
-                # footnote is always followed by a Null at root level.
-                if (trace_in_chunk and not last_trace_in_chunk and
-                        footnote_re.match(source_string_lines[last_trace_line_number-1])):
-                    in_footnote = True
-                    invalid_line_number = last_trace_line_number
-                    continue
-                if in_footnote:
-                    if trace_in_chunk:
-                        continue
-                    in_footnote = False
-                    while invalid_line_number < trace_line_number:
-                        invalid_line_numbers.add(invalid_line_number)
-                        invalid_line_number += 1
-                # Link definitions must be at root level and produce Nulls.
-                # Since they can contain much less text compared to footnotes,
-                # it is unlikely that they will significantly affect line
-                # numbers.  If more precision is needed later, something like
-                # approach below could be used.  However, it would need to be
-                # supplemented with a regex, like the footnote case, because
-                # the conditions below are also met by some raw nodes.
-                # ```
-                # if not trace_node_type and not last_trace_in_chunk and not trace_in_chunk:
-                #     invalid_line_number = last_trace_line_number
-                #     while invalid_line_number < trace_line_number:
-                #         invalid_line_numbers.add(invalid_line_number)
-                #         invalid_line_number += 1
-                # ```
-        except Exception as e:
-            raise PandocError('Incompatible Pandoc version or trace; cannot parse trace format:\n{0}'.format(e))
 
         # Iterator for source lines and line numbers that skips invalid lines
         if self.pandoc_file_scope or len(self.source_strings) == 1:
+            source_string_lines = util.splitlines_lf(source_string) or ['']
             source_name_line_and_number_iter = ((single_source_name, line, n+1)
-                                                for (n, line) in enumerate(source_string_lines)
-                                                if n+1 not in invalid_line_numbers)
+                                                for (n, line) in enumerate(source_string_lines))
         else:
             def make_source_name_line_and_number_iter():
-                line_and_concat_line_number_iter = ((line, n+1) for (n, line) in enumerate(source_string_lines))
                 for src_name, src_string in zip(self.source_names, self.source_strings):
-                    len_src_string_lines = src_string.count('\n')
-                    if src_string[-1:] != '\n':
-                        len_src_string_lines += 1
-                    for line_number in range(1, len_src_string_lines+1):
-                        line, concat_line_number = next(line_and_concat_line_number_iter)
-                        if concat_line_number in invalid_line_numbers:
-                            continue
-                        yield (src_name, line, line_number)
+                    src_string_lines = util.splitlines_lf(src_string) or ['']
+                    for n, line in enumerate(src_string_lines):
+                        yield (src_name, line, n+1)
             source_name_line_and_number_iter = make_source_name_line_and_number_iter()
 
 
@@ -1009,17 +983,21 @@ class PandocConverter(Converter):
                 pass
             elif node_type == 'Str':
                 node_contents = node['c']
-                line_index = line.find(node_contents, line_index)
-                if line_index >= 0:
-                    line_index += len(node_contents)
-                else:
-                    source_name, line, line_number = next(source_name_line_and_number_iter)
-                    line_index = line.find(node_contents)
-                    if line_index < 0:
-                        while line_index < 0:
+                for char in node_contents:
+                    if line[line_index:line_index+1] == char:
+                        line_index += 1
+                    else:
+                        line_index = line.find(char, line_index)
+                        if line_index >= 0:
+                            line_index += 1
+                        else:
                             source_name, line, line_number = next(source_name_line_and_number_iter)
-                            line_index = line.find(node_contents)
-                    line_index += len(node_contents)
+                            line_index = line.find(char)
+                            if line_index < 0:
+                                while line_index < 0:
+                                    source_name, line, line_number = next(source_name_line_and_number_iter)
+                                    line_index = line.find(char)
+                            line_index += 1
                 if para_plain_node is not None:
                     para_plain_source_name_node_line_number.append((source_name, para_plain_node, line_number))
                     para_plain_node = None
@@ -1037,7 +1015,7 @@ class PandocConverter(Converter):
                         line_index += len(node_contents)
                     else:
                         line_index = last_line_index
-                        for node_contents_elem in node_contents.split(' '):
+                        for node_contents_elem in node_contents.replace('\n', ' ').split(' '):
                             line_index = line.find(node_contents_elem, line_index)
                             if line_index >= 0:
                                 line_index += len(node_contents_elem)
@@ -1163,8 +1141,13 @@ class PandocConverter(Converter):
                     node['c'].insert(0, io_map_span_node(source_name, line_number))
 
         # Convert modified AST to markdown, then back, so that raw output
-        # can be reinterpreted as markdown
-        processed_to_format_extensions = self.from_format_pandoc_extensions or '' + '-latex_macros-smart'
+        # can be reinterpreted as markdown.
+        # Order of extensions is important: earlier override later.
+        processed_to_format_extensions = ''.join(['-latex_macros',
+                                                  '-raw_attribute',
+                                                  '-smart'])
+        if self.from_format_pandoc_extensions is not None:
+            processed_to_format_extensions += self.from_format_pandoc_extensions
         processed_markup = collections.OrderedDict()
         for source_name, ast in self._asts.items():
             markup_bytes, stderr_bytes = self._run_pandoc(input=json.dumps(ast),
@@ -1172,7 +1155,8 @@ class PandocConverter(Converter):
                                                           to_format='markdown',
                                                           to_format_pandoc_extensions=processed_to_format_extensions,
                                                           standalone=True,
-                                                          newline_lf=True)
+                                                          newline_lf=True,
+                                                          preserve_tabs=True)
             if stderr_bytes:
                 sys.stderr.buffer.write(stderr_bytes)
             processed_markup[source_name] = markup_bytes
@@ -1183,7 +1167,8 @@ class PandocConverter(Converter):
                                                                  from_format='markdown',
                                                                  from_format_pandoc_extensions=self.from_format_pandoc_extensions,
                                                                  to_format='json',
-                                                                 newline_lf=True)
+                                                                 newline_lf=True,
+                                                                 preserve_tabs=True)
                 if stderr_bytes:
                     sys.stderr.buffer.write(stderr_bytes)
         else:
@@ -1199,7 +1184,8 @@ class PandocConverter(Converter):
                                                                  from_format='markdown',
                                                                  from_format_pandoc_extensions=self.from_format_pandoc_extensions,
                                                                  to_format='json',
-                                                                 newline_lf=True)
+                                                                 newline_lf=True,
+                                                                 preserve_tabs=True)
                 if stderr_bytes:
                     sys.stderr.buffer.write(stderr_bytes)
         if sys.version_info < (3, 6):
@@ -1261,7 +1247,8 @@ class PandocConverter(Converter):
                                                              to_format_pandoc_extensions=to_format_pandoc_extensions,
                                                              standalone=standalone,
                                                              output_path=output_path,
-                                                             other_pandoc_args=other_pandoc_args)
+                                                             other_pandoc_args=other_pandoc_args,
+                                                             preserve_tabs=True)
             if stderr_bytes:
                 sys.stderr.buffer.write(stderr_bytes)
             if output_path is None:
@@ -1275,7 +1262,8 @@ class PandocConverter(Converter):
                                                              to_format_pandoc_extensions=to_format_pandoc_extensions,
                                                              standalone=standalone,
                                                              newline_lf=True,
-                                                             other_pandoc_args=other_pandoc_args)
+                                                             other_pandoc_args=other_pandoc_args,
+                                                             preserve_tabs=True)
             if stderr_bytes:
                 sys.stderr.buffer.write(stderr_bytes)
             converted_lines = util.splitlines_lf(converted_bytes.decode(encoding='utf8')) or ['']
